@@ -1,54 +1,23 @@
 # -*- coding: utf-8 -*-
-"""
-###############################################################################
-# Copyright 2014 Grigoriy Kramarenko.
-###############################################################################
-# This file is part of ReportAPI.
-#
-#    ReportAPI is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
-#
-#    ReportAPI is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with ReportAPI.  If not, see <http://www.gnu.org/licenses/>.
-#
-# Этот файл — часть ReportAPI.
-#
-#   ReportAPI - свободная программа: вы можете перераспространять ее и/или
-#   изменять ее на условиях Стандартной общественной лицензии GNU в том виде,
-#   в каком она была опубликована Фондом свободного программного обеспечения;
-#   либо версии 3 лицензии, либо (по вашему выбору) любой более поздней
-#   версии.
-#
-#   ReportAPI распространяется в надежде, что она будет полезной,
-#   но БЕЗО ВСЯКИХ ГАРАНТИЙ; даже без неявной гарантии ТОВАРНОГО ВИДА
-#   или ПРИГОДНОСТИ ДЛЯ ОПРЕДЕЛЕННЫХ ЦЕЛЕЙ. Подробнее см. в Стандартной
-#   общественной лицензии GNU.
-#
-#   Вы должны были получить копию Стандартной общественной лицензии GNU
-#   вместе с этой программой. Если это не так, см.
-#   <http://www.gnu.org/licenses/>.
-###############################################################################
-"""
+from __future__ import unicode_literals
+from django.utils.encoding import smart_text, python_2_unicode_compatible
+from django.utils import six
 from django.db import models
-from django.db.models import Q
-from django.utils.translation import ugettext_lazy as _
+from django.db.models import Q, get_model
+from django.utils.translation import ugettext_noop, ugettext_lazy as _
 from django.utils import timezone
-from django.template import Context
-from django.template.loader import get_template
+from django.template import RequestContext, loader
+from django.template.defaultfilters import slugify
 from django.core.files.base import ContentFile
 
 from reportapi.conf import (settings, REPORTAPI_CODE_HASHLIB,
     REPORTAPI_UPLOAD_HASHLIB, REPORTAPI_FILES_UNIDECODE,
-    AUTH_USER_MODEL, AUTH_GROUP_MODEL, REPORTAPI_CONVERTOR_BACKEND,
+    AUTH_USER_MODEL, REPORTAPI_CONVERTOR_BACKEND,
     REPORTAPI_PDFCONVERT_ARGS1, REPORTAPI_PDFCONVERT_ARGS2)
 
+User = get_model(*AUTH_USER_MODEL.split('.'))
+Group = User.groups.field.rel.to
+
 if REPORTAPI_FILES_UNIDECODE:
     from unidecode import unidecode
     prep_filename = lambda x: unidecode(x).replace(' ', '_').replace("'", "")
@@ -79,6 +48,12 @@ class Report(object):
     icon            = None
     template_name   = 'reportapi/docs/base.html'
     filters         = None
+    site            = None
+    name            = None
+    title           = None
+    verbose_name    = None
+    section         = ugettext_noop('main')
+    section_label   = None
 
     def __init__(self, site=None, section=None, section_label=None, \
         filters=None, title=None, name=None, **kwargs):
@@ -89,14 +64,14 @@ class Report(object):
 
         self.site = site or getattr(self, 'site', None) or raise_set_site(class_name)
 
-        self.section = section or getattr(self, 'section', 'main')
-        if not isinstance(self.section, (str, unicode)) or not validate_name(self.section):
+        self.section = section or getattr(self, 'section')
+        if not isinstance(self.section, six.string_types) or not validate_name(self.section):
             raise ValueError('Attribute `section` most be string in '
                 'English without digits, spaces and hyphens.')
-        self.section_label = section_label or getattr(self, 'section_label', _(self.section))
+        self.section_label = section_label or getattr(self, 'section_label', None) or _(self.section)
 
-        self.name = name or getattr(self, 'name', class_name.lower())
-        if not isinstance(self.name, (str, unicode)) or not validate_name(self.name):
+        self.name = name or getattr(self, 'name', None) or class_name.lower()
+        if not isinstance(self.name, six.string_types) or not validate_name(self.name):
             raise ValueError('Attribute `name` most be string in '
                 'English without digits, spaces and hyphens.')
 
@@ -111,7 +86,7 @@ class Report(object):
         self.filters = [ f for f in self.filters if f in self._filters.values() ]
 
         self.title = title or getattr(self, 'title')
-        if not isinstance(self.title, (str, unicode)) or not validate_title(self.title):
+        if not isinstance(self.title, six.string_types) or not validate_title(self.title):
             raise ValueError('Attribute `title` most be string in '
                 'English without translation.')
         self.verbose_name = _(self.title)
@@ -144,8 +119,8 @@ class Report(object):
         if user.is_superuser:
             return self.create_register()
 
-        registers = Register.objects.filter(section=self.section, name=self.name)
-        registers = registers.permitted(request)
+        registers = Register.objects.permitted(request)
+        registers = registers.filter(section=self.section, name=self.name)
         if registers:
             return registers[0]
         return None
@@ -153,29 +128,20 @@ class Report(object):
     def has_permission(self, request):
         return bool(self.permitted_register(request))
 
-    def filters_to_string(self, filters):
-        if not isinstance(filters, dict):
-            return str(filters)
-
-        for k,v in filters.items():
-            if isinstance(v, (list,tuple)):
-                filters[k] = list(set(v))
-        return str(filters)
-
     def get_code(self, request, filters):
         """
         Этот метод может быть переопределён для тех отчётов, где 
         уникальный код следует генерировать иначе.
         """
         if not filters:
-            return request.LANGUAGE_CODE
+            return getattr(request, 'LANGUAGE_CODE', settings.LANGUAGE_CODE)
         code = hashlib.new(REPORTAPI_CODE_HASHLIB)
         code.update(self.filters_to_string(filters))
-        code.update(request.LANGUAGE_CODE)
+        code.update(getattr(request, 'LANGUAGE_CODE', settings.LANGUAGE_CODE))
         return code.hexdigest()
 
     def get_filename(self):
-        filename = self.verbose_name + '.html'
+        filename = smart_text(self.verbose_name) + '.html'
         filename = prep_filename(filename)
         return filename
 
@@ -194,15 +160,57 @@ class Report(object):
         """
         context = self.get_context(request, document, filters)
         context['DOCUMENT'] = document
-        template = get_template(self.template_name)
-        content  = template.render(Context(context)).encode('utf-8')
-        _file    = ContentFile(content)
+        context['FILTERS'] = self.get_filters_data(filters)
+        content = loader.render_to_string(self.template_name, context,
+                            context_instance=RequestContext(request,))
+        _file = ContentFile(content.encode('utf-8') or \
+            smart_text(_('Unspecified render error in template.')))
         document.report_file.save(self.get_filename(), _file, save=save)
         return document
 
+    def filters_to_string(self, filters):
+        if not isinstance(filters, dict):
+            return str(filters)
+
+        for k,v in filters.items():
+            if isinstance(v, (list,tuple)):
+                filters[k] = list(set(v))
+        return smart_text(filters)
+
     def filters_list(self):
         return [ x.serialize() for x in self.filters ]
 
+    def validate_filters(self, filters):
+        try:
+            data = self.get_filters_data(filters)
+        except:
+            return False
+        return True
+
+    def get_filters_data(self, filters):
+        L = []
+        for key,dic in filters.items():
+            f = self.get_filter(key)
+            if f:
+                L.append(f.data(**dic))
+        return L
+
+    def get_filter_data(self, name, filters):
+        f = self.get_filter(name)
+        kw = filters.get(slugify(name), None)
+        if f and kw:
+            return f.data(**kw)
+        return None
+
+    def get_filter_clean_value(self, name, filters):
+        data = self.get_filter_data(name, filters)
+        if data:
+            return data.get('value', None)
+        return None
+
+    def get_filter(self, name):
+        return self._filters.get(slugify(name), None)
+
     def get_scheme(self, request=None):
         SCHEME = {
             'name': self.name,
@@ -213,6 +221,7 @@ class Report(object):
             'enable_threads': self.enable_threads,
             'create_force': self.create_force,
             'expiration_time': self.expiration_time,
+            'timeout': self.timeout,
         }
         filters_list = self.filters_list()
         SCHEME['filters'] = dict(filters_list)
@@ -220,11 +229,8 @@ class Report(object):
 
         return SCHEME
 
-    def get_filter(self, filter_name):
-        return self._filters.get(filter_name, None)
-
     @property
-    def timeout():
+    def timeout(self):
         return self.create_register().timeout
 
 class RegisterManager(models.Manager):
@@ -242,6 +248,7 @@ class RegisterManager(models.Manager):
             | Q(groups__in=user.groups.all())
         )
 
+@python_2_unicode_compatible
 class Register(models.Model):
     """
     Зарегистрированные отчёты.
@@ -251,11 +258,11 @@ class Register(models.Model):
     name    = models.CharField(_('name'), max_length=255)
     title = models.CharField(_('title without translation'), max_length=255)
     all_users = models.BooleanField(_('allow all users'), default=False)
-    users = models.ManyToManyField(AUTH_USER_MODEL, null=True, blank=True,
+    users = models.ManyToManyField(User, null=True, blank=True,
         verbose_name=_('allow list users'))
-    groups = models.ManyToManyField(AUTH_GROUP_MODEL, null=True, blank=True,
+    groups = models.ManyToManyField(Group, null=True, blank=True,
         verbose_name=_('allow list groups'))
-    timeout = models.IntegerField(_('max of timeout'), default=5000, editable=False)
+    timeout = models.IntegerField(_('max of timeout'), default=1000, editable=False)
 
     objects = RegisterManager()
 
@@ -265,9 +272,9 @@ class Register(models.Model):
         verbose_name_plural = _('registered reports')
         unique_together = ('section', 'name')
 
-    def __unicode__(self):
+    def __str__(self):
         try:
-            return unicode(_(self.title))
+            return smart_text(_(self.title))
         except:
             return self.title
 
@@ -290,6 +297,15 @@ class DocumentManager(models.Manager):
             | Q(register__groups__in=user.groups.all())
         )
 
+    def del_permitted(self, request):
+        user = request.user
+        if not user.is_authenticated():
+            return self.get_query_set().none()
+        if user.is_superuser:
+            return self.get_query_set().all()
+        return self.get_query_set().filter(user=user).all()
+
+@python_2_unicode_compatible
 class Document(models.Model):
     """
     Файл сформированного документа.
@@ -302,7 +318,7 @@ class Document(models.Model):
     """
     register = models.ForeignKey(Register, editable=False,
         verbose_name=_('registered report'))
-    user = models.ForeignKey(AUTH_USER_MODEL, editable=False, null=True,
+    user = models.ForeignKey(User, editable=False, null=True,
         verbose_name=_('user'))
     code  = models.CharField(_('process key'), editable=False,
         blank=True, db_index=True, max_length=REPORTAPI_CODE_LENGTH)
@@ -314,11 +330,11 @@ class Document(models.Model):
 
     objects = DocumentManager()
 
-    def __unicode__(self):
-        return unicode(self.register)
+    def __str__(self):
+        return smart_text(self.register)
 
     class Meta:
-        ordering = ['-end', '-start']
+        ordering = ['-start', '-end']
         verbose_name = _('generated report')
         verbose_name_plural = _('generated reports')
 
@@ -333,7 +349,7 @@ class Document(models.Model):
         code.update(str(dt.isoformat()))
         code.update('reportapi'+settings.SECRET_KEY)
         dic['code'] = code.hexdigest()
-        return u'reports/%(date)s/%(code)s/%(filename)s' % dic
+        return smart_text('reports/%(date)s/%(code)s/%(filename)s' % dic)
 
     @models.permalink
     def get_absolute_url(self):
@@ -375,7 +391,7 @@ class Document(models.Model):
         path = self.report_file.path
         newpath = path[:-4] + format # cut 'html' and append format file
         if format in ('odt', 'ods'):
-            if not exists(newpath):
+            if not os.path.exists(newpath):
                 cwd = os.getcwd()
                 os.chdir(os.path.dirname(path))
                 os.symlink(os.path.basename(path), os.path.basename(newpath))
@@ -423,7 +439,8 @@ class Document(models.Model):
         super(Document, self).save()
 
     def delete(self):
-        remove_file(self.report_file.path)
+        if self.report_file:
+            remove_file(self.report_file.path)
         super(Document, self).delete()
 
 
