@@ -1,45 +1,46 @@
 /* reportapi.js for ReportAPI
-###############################################################################
-# Copyright 2014 Grigoriy Kramarenko.
-###############################################################################
-# This file is part of ReportAPI.
-#
-#    ReportAPI is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
-#
-#    ReportAPI is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
-#
-#    You should have received a copy of the GNU General Public License
-#    along with ReportAPI.  If not, see <http://www.gnu.org/licenses/>.
-#
-# Этот файл — часть ReportAPI.
-#
-#   ReportAPI - свободная программа: вы можете перераспространять ее и/или
-#   изменять ее на условиях Стандартной общественной лицензии GNU в том виде,
-#   в каком она была опубликована Фондом свободного программного обеспечения;
-#   либо версии 3 лицензии, либо (по вашему выбору) любой более поздней
-#   версии.
-#
-#   ReportAPI распространяется в надежде, что она будет полезной,
-#   но БЕЗО ВСЯКИХ ГАРАНТИЙ; даже без неявной гарантии ТОВАРНОГО ВИДА
-#   или ПРИГОДНОСТИ ДЛЯ ОПРЕДЕЛЕННЫХ ЦЕЛЕЙ. Подробнее см. в Стандартной
-#   общественной лицензии GNU.
-#
-#   Вы должны были получить копию Стандартной общественной лицензии GNU
-#   вместе с этой программой. Если это не так, см.
-#   <http://www.gnu.org/licenses/>.
-###############################################################################
-*/
+ * 
+ * Copyright 2014 Grigoriy Kramarenko.
+ * 
+ * This file is part of ReportAPI.
+ * 
+ * ReportAPI is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * ReportAPI is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with ReportAPI.  If not, see <http://www.gnu.org/licenses/>.
+ * 
+ * Этот файл — часть ReportAPI.
+ * 
+ * ReportAPI - свободная программа: вы можете перераспространять ее и/или
+ * изменять ее на условиях Стандартной общественной лицензии GNU в том виде,
+ * в каком она была опубликована Фондом свободного программного обеспечения;
+ * либо версии 3 лицензии, либо (по вашему выбору) любой более поздней
+ * версии.
+ * 
+ * ReportAPI распространяется в надежде, что она будет полезной,
+ * но БЕЗО ВСЯКИХ ГАРАНТИЙ; даже без неявной гарантии ТОВАРНОГО ВИДА
+ * или ПРИГОДНОСТИ ДЛЯ ОПРЕДЕЛЕННЫХ ЦЕЛЕЙ. Подробнее см. в Стандартной
+ * общественной лицензии GNU.
+ *
+ * Вы должны были получить копию Стандартной общественной лицензии GNU
+ * вместе с этой программой. Если это не так, см.
+ * <http://www.gnu.org/licenses/>.
+ * 
+ */
 
 ////////////////////////////////////////////////////////////////////////
 //                   КОНСТАНТЫ И ГЛОБАЛЬНЫЕ ПЕРЕМЕННЫЕ                //
 ////////////////////////////////////////////////////////////////////////
-var TIMEOUT_PROGRESS = 1000;
+var TIMEOUT_PROGRESS = 1000,
+    SERVER_TZ_OFFSET = window.SERVER_TZ_OFFSET || -240; // Europe/Moscow
 
 // Глобальные хранилища-регистраторы
 window.TEMPLATES = {}; // Шаблоны
@@ -61,15 +62,6 @@ _.mixin(_.str.exports());
 //                               ОБЩИЕ                                //
 ////////////////////////////////////////////////////////////////////////
 
-/* Проверка объекта на пустоту */
-function isEmpty(obj) {
-    for (var k in obj) {
-        return false; // если цикл хоть раз сработал, то объект не пустой => false
-    };
-    // дошли до этой строки - значит цикл не нашёл ни одного свойства => true
-    return true;
-};
-
 /* Единая, переопределяемая задержка для действий или функций */
 delay = (function(){
     var timer = 0;
@@ -80,41 +72,6 @@ delay = (function(){
     };
 })();
 
-/* Генератор идентификаторов, которому можно задавать статические
- * начало и конец идентификатора, например:
- *  >> id = generatorID()
- *  >> "i1363655293735"
- *  >> id = generatorID(null, "object")
- *  >> "gen1363655293736_object"
- *  >> id = generatorID("object")
- *  >> "object_i1363655293737"
- *  >> id = generatorID("model", "object")
- *  >> "model_i1363655293738_object"
- */
-function generatorID(prefix, postfix) {
-    if (DEBUG) {console.log('function:'+'generatorID')};
-    var result = [],
-        gen = 'i',
-        m = 1000,
-        n = 9999,
-        salt = Math.floor( Math.random() * (n - m + 1) ) + m;
-    gen += $.now() + String(salt);
-    if (prefix) { result.push(prefix)};
-    result.push(gen); 
-    if (postfix) { result.push(postfix) };
-    return validatorID(result);
-};
-
-/* Приводит идентификаторы в позволительный jQuery вид.
- * В данном приложении он заменяет точки на "-".
- * На вход может принимать список или строку
- */
-function validatorID(id) {
-    if ($.type(id) === 'array') { id = id.join('_'); };
-    if (DEBUG) { console.log('function:'+'validatorID'); };
-    return id.replace(/[\.,\:,\/, ,\(,\),=,?]/g, "-");
-};
-
 /* Общие функции вывода сообщений */
 function handlerHideAlert() {
     if (DEBUG) {console.log('function:'+'handlerHideAlert')};
@@ -209,24 +166,28 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
 /* Обработчик создания отчёта */
 function handlerCreateReport(force) {
     if (DEBUG) {console.log('function:'+'handlerCreateReport')};
-    filters = {};
+    $('.action-preview, .action-remove, .action-download')
+        .attr('disabled', 'disabled');
+    window.REPORT.id = null;
+    window.REPORT.process_filters = {};
     $.each(window.REPORT.filters, function(key, item) {
         if (item.condition || item.value != null) {
-            filters[item.name] = {condition:item.condition, value:item.value};
+            window.REPORT.process_filters[item.name] = {condition:item.condition, value:item.value};
         };
     });
     args = {
-        'method': 'reportapi.create_document',
+        'method': 'reportapi.document_create',
         'section': window.REPORT.section,
         'name': window.REPORT.name,
-        'filters': filters,
+        'filters': window.REPORT.process_filters,
         'force': force,
     };
     cb = function(json, status, xhr) {
-        handlerStartProgress(json.data);
+        
     };
-    
-    var jqxhr = jsonAPI(args, cb);
+
+    var jqxhr = jsonAPI(args, cb, null, false, window.REPORT.timeout);
+    handlerStartProgress();
     return jqxhr;
 };
 
@@ -247,24 +208,48 @@ function handlerCheckProcess() {
     if (DEBUG) {console.log('function:'+'handlerCheckProcess')};
     args = {
         'method': 'reportapi.document_info',
-        'id': window.REPORT.id,
+        'id': window.REPORT.id || null,
+        'section': window.REPORT.section,
+        'name': window.REPORT.name,
+        'filters': window.REPORT.process_filters,
     };
+
     cb = function(json, status, xhr) {
         $obj = $('.progress .progress-bar');
         max = Number($obj.attr('aria-valuemax'));
         now = Number($obj.attr('aria-valuetransitiongoal'));
+        window.REPORT.id = json.data.id;
+        window.REPORT.timeout = json.data.timeout;
         if (json.data.end) {
             clearInterval(window.REPORT.process);
             window.REPORT.process = undefined;
             $obj.attr('aria-valuetransitiongoal', max).progressbar();
             
             url = json.data.url + window.REPORT.format.toLowerCase() + '/';
-            $('.action-download')
-                .attr('href', url)
-                .prop('disabled', false).show();
+
             $('.action-preview')
                 .attr('onClick', "handlerShowDocument('"+json.data.url+"', 'document-"+json.data.id+"')")
-                .prop('disabled', false).show();
+                .removeAttr('disabled').show();
+            if (json.data.error) {
+                $('.action-download').attr('disabled', 'disabled').hide();
+                $('.action-remove')
+                    .attr('onClick', "handlerRemoveDocument("+json.data.id+")")
+                    .attr('disabled', 'disabled').show();
+                if (json.data.has_remove) {
+                    $('.action-remove').removeAttr('disabled');
+                };
+                $('.action-preview')
+                    .removeClass('btn-default').addClass('btn-warning')
+                    .find('.fa').removeClass('fa-search').addClass('fa-bug');
+            } else {
+                $('.action-remove').attr('disabled', 'disabled').hide();
+                $('.action-download')
+                    .attr('href', url)
+                    .removeAttr('disabled').show();
+                $('.action-preview')
+                    .removeClass('btn-warning').addClass('btn-default')
+                    .find('.fa').removeClass('fa-bug').addClass('fa-search');
+            };
             $('.progress')
                 .removeClass('progress-striped')
                 .removeClass('active')
@@ -287,29 +272,50 @@ function handlerCheckProcess() {
             $obj.attr('aria-valuetransitiongoal', TIMEOUT_PROGRESS + now).progressbar();
         };
     };
-    
+
     var jqxhr = jsonAPI(args, cb);
     return jqxhr;
 };
 
-
 /* Обработчик запуска ожидания создания отчёта */
-function handlerStartProgress(data) {
+function handlerStartProgress() {
     if (DEBUG) {console.log('function:'+'handlerStartProgress')};
     $('.action-create-report').prop("disabled", true);
     $('.progress .progress-bar')
-        .attr('aria-valuemax', data.timeout || 5000)
-        .attr('aria-valuetransitiongoal', 0)
+        .attr('aria-valuemax', window.REPORT.timeout || TIMEOUT_PROGRESS)
+        .attr('aria-valuetransitiongoal', 1)
         .progressbar();
     $('.progress').show();
     
-    window.REPORT.id = data.id;
-    window.REPORT.timeout = data.timeout;
     window.REPORT.process = setInterval(function () {
         handlerCheckProcess();
     }, window.TIMEOUT_PROGRESS);
 };
 
+/* Обработчик удаления отчёта */
+function handlerRemoveDocument(id) {
+    if (DEBUG) {console.log('function:'+'handlerRemoveDocument')};
+    if (id) {
+        var args = {
+                'method': 'reportapi.document_delete',
+                'id': id,
+            },
+            success = function() {
+                $('.action-preview, .action-remove, .action-download')
+                    .attr('disabled', 'disabled').hide();
+                $('#thumbnail-document-'+id).remove();
+            };
+        new jsonAPI(args, success);
+    };
+    
+};
+
+/* Обработчик события удаления отчёта */
+function eventRemoveDocument(event) {
+    if (DEBUG) {console.log('function:'+'eventRemoveDocument')};
+    var id = null;
+    handlerRemoveDocument(id);
+};
 
 ////////////////////////////////////////////////////////////////////////
 //                    Обработчики типов фильтров                      //
@@ -321,18 +327,16 @@ function handlerSetSelectizers($box) {
     $box = $box || $('body');
 
     $.each($box.find('select[data-type="object"]'), function(index, select) {
-        data = $(select).data() || {};
-        type = data.type;
-        filter_name = select.name;
 
-        if (filter_name) {
-            filter = window.REPORT.filters[filter_name];
-            unicode_key = filter.unicode_key || '__unicode__';
+        if (select.name) {
+            var filter = window.REPORT.filters[select.name],
+                unicode_key = filter.unicode_key || '__unicode__';
             $(select).selectize({
                 valueField: 'pk',
                 labelField: unicode_key,
-                searchField: unicode_key,
+                searchField: filter.fields_search || unicode_key,
                 create: false,
+                options: filter.options,
                 maxItems: (filter.condition == 'range') ? 2 : undefined,
                 render: {
                     option: function(item, escape) {
@@ -343,36 +347,66 @@ function handlerSetSelectizers($box) {
                     }
                 },
                 load: function(query, callback) {
-                    //~ if (!query.length) return callback();
+                    if (!query.length) return callback();
+                    if (filter.search_on_date) {
+                        var day = moment(query);
+                        if (day.invalidAt() > -1 || query.length <10) {
+                            return callback();
+                        };
+                    };
 
-                    $.ajax({
-                        type: "POST",
-                        timeout: window.AJAX_TIMEOUT,
-                        url: REPORTAPI_API_URL,
-                        data: {'jsonData': $.toJSON({
+                    var args = {
                             'method': "reportapi.object_search",
                             'section': window.REPORT.section || null,
                             'name': window.REPORT.name,
                             'filter_name': filter_name,
                             'query': query,
-                        }), 'language': window.LANGUAGE_CODE},
-                        dataType: 'json',
-                        error: function() {
-                            callback();
                         },
-                        success: function(res) {
-                            console.log(res);
+                        success = function(res) {
+                            //~ console.log(res);
                             callback(res.data.object_list);
-                        }
-                    });
+                        };
+                    new jsonAPI(args, success);
                 },
                 onChange: function(value) {
-                    filter.value = value || null;
-                    handlerCheckRequiredValue();
+                    if (filter.condition == 'range' && value.length != 2) {
+                        filter.value = null;
+                    } else {
+                        filter.value = (value == '') ? null : value;
+                    };
+                    handlerAfterChanges();
                 },
                 onClear: function() {
                     filter.value = null;
-                    handlerCheckRequiredValue();
+                    handlerAfterChanges();
+                }
+            });
+        }
+    });
+
+    other = $box.find('select[data-type="choice"], select[data-type="month"], select[data-type="weekday"]');
+    $.each(other, function(index, select) {
+
+        if (select.name) {
+            var filter = window.REPORT.filters[select.name];
+            $(select).selectize({
+                options: filter.options,
+                valueField: 'value',
+                labelField: 'label',
+                searchField: 'label',
+                create: false,
+                maxItems: (filter.condition == 'range') ? 2 : undefined,
+                onChange: function(value) {
+                    if (filter.condition == 'range' && value.length != 2) {
+                        filter.value = null;
+                    } else {
+                        filter.value = (value == '') ? null : value;
+                    };
+                    handlerAfterChanges();
+                },
+                onClear: function() {
+                    filter.value = null;
+                    handlerAfterChanges();
                 }
             });
         }
@@ -394,10 +428,10 @@ function handlerMaskInputs($box) {
         data = $(item).data();
         $item.mask(data.mask, {
             completed: function() {
-                report = REPORT.filters[this.context.name];
-                report.value = this.val();
-                if (report.condition == 'range') {
-                    report.value = report.value.split(RANGE_SPLIT);
+                filter = REPORT.filters[this.context.name];
+                filter.value = this.val();
+                if (filter.condition == 'range') {
+                    filter.value = filter.value.split(RANGE_SPLIT);
                 };
             }
         });
@@ -424,28 +458,44 @@ function handlerCheckRequiredValue() {
     return completed;
 };
 
-// События
-
-/* Обработчик события сброса значения фильтра с маской */
-function eventChangeValue(event) {
-    if (DEBUG) {console.log('function:'+'eventChangeValue')};//, event)};
-    value = $(event.target).val();
-    report = REPORT.filters[event.target.name];
-    if (event.target.type == 'checkbox') {
-        report.value = event.target.checked;
-    } else {
-        report.value = value || null;
+/* Постобработчик изменения значения или условия фильтра */
+function handlerAfterChanges() {
+    if (DEBUG) {console.log('function:'+'handlerAfterChanges')};
+    if (window.REPORT.process) {
+        clearInterval(window.REPORT.process);
+        window.REPORT.id = null;
+        window.REPORT.process = null;
     };
     $('.progress:visible').hide();
-    $('.progress:visible .progress-bar').attr('aria-valuetransitiongoal', 0)
+    $('.progress .progress-bar').attr('aria-valuetransitiongoal', 0)
         .progressbar();
     $('.action-preview:visible').hide();
     $('.action-download:visible').hide();
     $('.action-recreate-report:visible').hide();
     $('.action-create-report:hidden').show();
+    $('.action-preview, .action-remove, .action-download')
+        .attr('disabled', 'disabled').hide();
     handlerCheckRequiredValue();
 };
 
+// События
+
+
+/* Обработчик события изменения значения фильтра */
+function eventChangeValue(event) {
+    if (DEBUG) {console.log('function:'+'eventChangeValue')};//, event)};
+    value = $(event.target).val();
+    filter = REPORT.filters[event.target.name];
+    if (event.target.type == 'checkbox') {
+        filter.value = event.target.checked;
+    } else if ($(event.target).attr('data-mask') !== undefined) {
+        filter.value = value ? filter.value : null;
+    } else {
+        filter.value = value || null;
+    };
+    handlerAfterChanges();
+};
+
 /* Обработчик события изменения условия фильтра */
 function eventConditionChange(event) {
     if (DEBUG) {console.log('function:'+'eventConditionChange')};//, event)};
@@ -453,10 +503,10 @@ function eventConditionChange(event) {
     filter_name = $(event.target).data()['name'];
     filter = REPORT.filters[filter_name];
     filter.condition = event.target.value || null;
-    console.log(filter.condition);
+    //~ console.log(filter.condition);
     if (filter.condition in {'isnull':0, 'empty':0}) {
         filter.value = false;
-    } else if (!filter.condition || filter.type == 'object') {
+    } else if (!filter.condition || filter.condition in {'range':0,'in':0} || filter.type == 'object') {
         filter.value = null;
     } else {
         filter.value = $('#value-'+filter_name).val() || null;
@@ -466,14 +516,7 @@ function eventConditionChange(event) {
     handlerSetSelectizers($('#valuebox-'+filter_name));
     handlerMaskInputs($('#valuebox-'+filter_name));
 
-    $('.progress:visible').hide();
-    $('.progress:visible .progress-bar').attr('aria-valuetransitiongoal', 0)
-        .progressbar();
-    $('.action-preview:visible').hide();
-    $('.action-download:visible').hide();
-    $('.action-recreate-report:visible').hide();
-    $('.action-create-report:hidden').show();
-    handlerCheckRequiredValue();
+    handlerAfterChanges();
 
     return true;
 };
@@ -498,7 +541,7 @@ function eventKeyDownOnDateTime(event) {
 
     prepare = function(event) {
         value = event.target.value || '';
-        console.log(value, event.charCode);
+        //~ console.log(value, event.charCode);
     };
 
     // Enabled keys browser control
@@ -552,13 +595,6 @@ function handlerShowDocument(href, name) {
     win.focus();
 };
 
-/* Обработчик просмотра отчёта */
-function handlerShowDocument(href, name) {
-    if (DEBUG) {console.log('function:'+'handlerShowDocument')};
-    var win = window.open(href, name, 'height=500,width=800,resizable=yes,scrollbars=yes');
-    win.focus();
-};
-
 /* Обработчик отображения дополнительных фильтров */
 function handlerShowAdditionalFilters() {
     if (DEBUG) {console.log('function:'+'handlerShowAdditionalFilters')};
@@ -630,6 +666,8 @@ $(document).ready(function($) {
     // Установка биндингов для элементов
     handlerBindinds();
 
+    // Прочее
+    handlerMaskInputs();
     handlerHideAdditionalFilters();
 
 });
